@@ -33,17 +33,6 @@ macro_rules! ffi_dyn {
     }};
 }
 
-macro_rules! ffi_dict_dyn {
-    ($array:expr, $ty:ty) => {{
-        let array = $array.as_any().downcast_ref::<$ty>().unwrap();
-        (
-            array.buffers(),
-            array.children(),
-            Some(array.values().clone()),
-        )
-    }};
-}
-
 type BuffersChildren = (
     Vec<Option<std::ptr::NonNull<u8>>>,
     Vec<Arc<dyn Array>>,
@@ -85,17 +74,16 @@ pub fn buffers_children_dictionary(array: &dyn Array) -> BuffersChildren {
         DataType::LargeList(_) => ffi_dyn!(array, ListArray::<i64>),
         DataType::FixedSizeList(_, _) => ffi_dyn!(array, FixedSizeListArray),
         DataType::Struct(_) => ffi_dyn!(array, StructArray),
-        DataType::Union(_) => unimplemented!(),
-        DataType::Dictionary(key_type, _) => match key_type.as_ref() {
-            DataType::Int8 => ffi_dict_dyn!(array, DictionaryArray::<i8>),
-            DataType::Int16 => ffi_dict_dyn!(array, DictionaryArray::<i16>),
-            DataType::Int32 => ffi_dict_dyn!(array, DictionaryArray::<i32>),
-            DataType::Int64 => ffi_dict_dyn!(array, DictionaryArray::<i64>),
-            DataType::UInt8 => ffi_dict_dyn!(array, DictionaryArray::<u8>),
-            DataType::UInt16 => ffi_dict_dyn!(array, DictionaryArray::<u16>),
-            DataType::UInt32 => ffi_dict_dyn!(array, DictionaryArray::<u32>),
-            DataType::UInt64 => ffi_dict_dyn!(array, DictionaryArray::<u64>),
-            _ => unreachable!(),
-        },
+        DataType::Union(_, _, _) => ffi_dyn!(array, UnionArray),
+        DataType::Dictionary(key_type, _) => {
+            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+                let array = array.as_any().downcast_ref::<DictionaryArray<$T>>().unwrap();
+                (
+                    array.buffers(),
+                    array.children(),
+                    Some(array.values().clone()),
+                )
+            })
+        }
     }
 }
