@@ -1,19 +1,15 @@
-//! This module contains arrays: fixed-length and immutable containers with optional values
+//! fixed-length and immutable containers with optional values
 //! that are layed in memory according to the Arrow specification.
 //! Each array type has its own `struct`. The following are the main array types:
-//!
 //! * [`PrimitiveArray`], an array of values with a fixed length such as integers, floats, etc.
 //! * [`BooleanArray`], an array of boolean values (stored as a bitmap)
 //! * [`Utf8Array`], an array of utf8 values
 //! * [`BinaryArray`], an array of binary values
 //! * [`ListArray`], an array of arrays (e.g. `[[1, 2], None, [], [None]]`)
 //! * [`StructArray`], an array of arrays identified by a string (e.g. `{"a": [1, 2], "b": [true, false]}`)
-//!
-//! This module contains constructors and accessors to operate on the arrays.
-//! All the arrays implement the trait [`Array`] and are often trait objects.
-//! Every array has a [`DataType`], which you can access with [`Array::data_type`].
-//! This can be used to `downcast_ref` a `&dyn Array` to a concrete struct.
-//! Arrays can share memory via [`crate::buffer::Buffer`] and thus cloning and slicing is `O(1)`.
+//! All arrays implement the trait [`Array`] and are often trait objects that can be downcasted
+//! to a concrete struct based on [`DataType`] available from [`Array::data_type`].
+//! Arrays share memory via [`crate::buffer::Buffer`] and thus cloning and slicing them `O(1)`.
 //!
 //! This module also contains the mutable counterparts of arrays, that are neither clonable nor slicable, but that
 //! can be operated in-place, such as [`MutablePrimitiveArray`] and [`MutableUtf8Array`].
@@ -56,6 +52,9 @@ pub trait Array: std::fmt::Debug + Send + Sync {
     /// This is `O(1)`.
     #[inline]
     fn null_count(&self) -> usize {
+        if self.data_type() == &DataType::Null {
+            return self.len();
+        };
         self.validity()
             .as_ref()
             .map(|x| x.null_count())
@@ -147,6 +146,23 @@ macro_rules! fmt_dyn {
     }};
 }
 
+macro_rules! with_match_dictionary_key_type {(
+    $key_type:expr, | $_:tt $T:ident | $($body:tt)*
+) => ({
+    macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    match $key_type {
+        DataType::Int8 => __with_ty__! { i8 },
+        DataType::Int16 => __with_ty__! { i16 },
+        DataType::Int32 => __with_ty__! { i32 },
+        DataType::Int64 => __with_ty__! { i64 },
+        DataType::UInt8 => __with_ty__! { u8 },
+        DataType::UInt16 => __with_ty__! { u16 },
+        DataType::UInt32 => __with_ty__! { u32 },
+        DataType::UInt64 => __with_ty__! { u64 },
+        _ => ::core::unreachable!("A dictionary key type can only be of integer types"),
+    }
+})}
+
 impl Display for dyn Array {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self.data_type() {
@@ -185,18 +201,12 @@ impl Display for dyn Array {
             DataType::LargeList(_) => fmt_dyn!(self, ListArray::<i64>, f),
             DataType::FixedSizeList(_, _) => fmt_dyn!(self, FixedSizeListArray, f),
             DataType::Struct(_) => fmt_dyn!(self, StructArray, f),
-            DataType::Union(_) => unimplemented!(),
-            DataType::Dictionary(key_type, _) => match key_type.as_ref() {
-                DataType::Int8 => fmt_dyn!(self, DictionaryArray::<i8>, f),
-                DataType::Int16 => fmt_dyn!(self, DictionaryArray::<i16>, f),
-                DataType::Int32 => fmt_dyn!(self, DictionaryArray::<i32>, f),
-                DataType::Int64 => fmt_dyn!(self, DictionaryArray::<i64>, f),
-                DataType::UInt8 => fmt_dyn!(self, DictionaryArray::<u8>, f),
-                DataType::UInt16 => fmt_dyn!(self, DictionaryArray::<u16>, f),
-                DataType::UInt32 => fmt_dyn!(self, DictionaryArray::<u32>, f),
-                DataType::UInt64 => fmt_dyn!(self, DictionaryArray::<u64>, f),
-                _ => unreachable!(),
-            },
+            DataType::Union(_, _, _) => fmt_dyn!(self, UnionArray, f),
+            DataType::Dictionary(key_type, _) => {
+                with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+                    fmt_dyn!(self, DictionaryArray::<$T>, f)
+                })
+            }
         }
     }
 }
@@ -239,23 +249,18 @@ pub fn new_empty_array(data_type: DataType) -> Box<dyn Array> {
         DataType::LargeList(_) => Box::new(ListArray::<i64>::new_empty(data_type)),
         DataType::FixedSizeList(_, _) => Box::new(FixedSizeListArray::new_empty(data_type)),
         DataType::Struct(fields) => Box::new(StructArray::new_empty(&fields)),
-        DataType::Union(_) => unimplemented!(),
-        DataType::Dictionary(key_type, value_type) => match key_type.as_ref() {
-            DataType::Int8 => Box::new(DictionaryArray::<i8>::new_empty(*value_type)),
-            DataType::Int16 => Box::new(DictionaryArray::<i16>::new_empty(*value_type)),
-            DataType::Int32 => Box::new(DictionaryArray::<i32>::new_empty(*value_type)),
-            DataType::Int64 => Box::new(DictionaryArray::<i64>::new_empty(*value_type)),
-            DataType::UInt8 => Box::new(DictionaryArray::<u8>::new_empty(*value_type)),
-            DataType::UInt16 => Box::new(DictionaryArray::<u16>::new_empty(*value_type)),
-            DataType::UInt32 => Box::new(DictionaryArray::<u32>::new_empty(*value_type)),
-            DataType::UInt64 => Box::new(DictionaryArray::<u64>::new_empty(*value_type)),
-            _ => unreachable!(),
-        },
+        DataType::Union(_, _, _) => Box::new(UnionArray::new_empty(data_type)),
+        DataType::Dictionary(key_type, value_type) => {
+            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+                Box::new(DictionaryArray::<$T>::new_empty(*value_type))
+            })
+        }
     }
 }
 
 /// Creates a new [`Array`] of [`DataType`] `data_type` and `length`.
-/// The array is guaranteed to have [`Array::null_count`] equal to [`Array::len`].
+/// The array is guaranteed to have [`Array::null_count`] equal to [`Array::len`]
+/// for all types except Union, which does not have a validity.
 pub fn new_null_array(data_type: DataType, length: usize) -> Box<dyn Array> {
     match data_type {
         DataType::Null => Box::new(NullArray::new_null(length)),
@@ -293,18 +298,12 @@ pub fn new_null_array(data_type: DataType, length: usize) -> Box<dyn Array> {
         DataType::LargeList(_) => Box::new(ListArray::<i64>::new_null(data_type, length)),
         DataType::FixedSizeList(_, _) => Box::new(FixedSizeListArray::new_null(data_type, length)),
         DataType::Struct(fields) => Box::new(StructArray::new_null(&fields, length)),
-        DataType::Union(_) => unimplemented!(),
-        DataType::Dictionary(key_type, value_type) => match key_type.as_ref() {
-            DataType::Int8 => Box::new(DictionaryArray::<i8>::new_null(*value_type, length)),
-            DataType::Int16 => Box::new(DictionaryArray::<i16>::new_null(*value_type, length)),
-            DataType::Int32 => Box::new(DictionaryArray::<i32>::new_null(*value_type, length)),
-            DataType::Int64 => Box::new(DictionaryArray::<i64>::new_null(*value_type, length)),
-            DataType::UInt8 => Box::new(DictionaryArray::<u8>::new_null(*value_type, length)),
-            DataType::UInt16 => Box::new(DictionaryArray::<u16>::new_null(*value_type, length)),
-            DataType::UInt32 => Box::new(DictionaryArray::<u32>::new_null(*value_type, length)),
-            DataType::UInt64 => Box::new(DictionaryArray::<u64>::new_null(*value_type, length)),
-            _ => unreachable!(),
-        },
+        DataType::Union(_, _, _) => Box::new(UnionArray::new_null(data_type, length)),
+        DataType::Dictionary(key_type, value_type) => {
+            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+                Box::new(DictionaryArray::<$T>::new_null(*value_type, length))
+            })
+        }
     }
 }
 
@@ -354,18 +353,12 @@ pub fn clone(array: &dyn Array) -> Box<dyn Array> {
         DataType::LargeList(_) => clone_dyn!(array, ListArray::<i64>),
         DataType::FixedSizeList(_, _) => clone_dyn!(array, FixedSizeListArray),
         DataType::Struct(_) => clone_dyn!(array, StructArray),
-        DataType::Union(_) => unimplemented!(),
-        DataType::Dictionary(key_type, _) => match key_type.as_ref() {
-            DataType::Int8 => clone_dyn!(array, DictionaryArray::<i8>),
-            DataType::Int16 => clone_dyn!(array, DictionaryArray::<i16>),
-            DataType::Int32 => clone_dyn!(array, DictionaryArray::<i32>),
-            DataType::Int64 => clone_dyn!(array, DictionaryArray::<i64>),
-            DataType::UInt8 => clone_dyn!(array, DictionaryArray::<u8>),
-            DataType::UInt16 => clone_dyn!(array, DictionaryArray::<u16>),
-            DataType::UInt32 => clone_dyn!(array, DictionaryArray::<u32>),
-            DataType::UInt64 => clone_dyn!(array, DictionaryArray::<u64>),
-            _ => unreachable!(),
-        },
+        DataType::Union(_, _, _) => clone_dyn!(array, UnionArray),
+        DataType::Dictionary(key_type, _) => {
+            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+                clone_dyn!(array, DictionaryArray::<$T>)
+            })
+        }
     }
 }
 
@@ -380,6 +373,7 @@ mod null;
 mod primitive;
 mod specification;
 mod struct_;
+mod union;
 mod utf8;
 
 mod equal;
@@ -388,17 +382,19 @@ pub mod growable;
 pub mod ord;
 
 pub use display::get_display;
+pub use equal::equal;
 
 pub use binary::{BinaryArray, MutableBinaryArray};
 pub use boolean::{BooleanArray, MutableBooleanArray};
 pub use dictionary::{DictionaryArray, DictionaryKey, MutableDictionaryArray};
-pub use fixed_size_binary::FixedSizeBinaryArray;
-pub use fixed_size_list::FixedSizeListArray;
+pub use fixed_size_binary::{FixedSizeBinaryArray, MutableFixedSizeBinaryArray};
+pub use fixed_size_list::{FixedSizeListArray, MutableFixedSizeListArray};
 pub use list::{ListArray, MutableListArray};
 pub use null::NullArray;
 pub use primitive::*;
 pub use specification::Offset;
 pub use struct_::StructArray;
+pub use union::UnionArray;
 pub use utf8::{MutableUtf8Array, Utf8Array, Utf8ValuesIter};
 
 pub(crate) use self::ffi::buffers_children_dictionary;
@@ -411,6 +407,12 @@ pub trait TryExtend<A> {
     fn try_extend<I: IntoIterator<Item = A>>(&mut self, iter: I) -> Result<()>;
 }
 
+/// A trait describing the ability of a struct to receive new items.
+pub trait TryPush<A> {
+    /// Tries to push a new element.
+    fn try_push(&mut self, item: A) -> Result<()>;
+}
+
 fn display_helper<T: std::fmt::Display, I: IntoIterator<Item = Option<T>>>(iter: I) -> Vec<String> {
     let iterator = iter.into_iter();
     let len = iterator.size_hint().0;
@@ -470,55 +472,6 @@ pub unsafe trait GenericBinaryArray<O: Offset>: Array {
     fn offsets(&self) -> &[O];
 }
 
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use crate::datatypes::*;
-
-    #[test]
-    fn nulls() {
-        let datatypes = vec![
-            DataType::Int32,
-            DataType::Float64,
-            DataType::Utf8,
-            DataType::Binary,
-            DataType::List(Box::new(Field::new("a", DataType::Binary, true))),
-        ];
-        let a = datatypes
-            .into_iter()
-            .all(|x| new_null_array(x, 10).null_count() == 10);
-        assert!(a);
-    }
-
-    #[test]
-    fn empty() {
-        let datatypes = vec![
-            DataType::Int32,
-            DataType::Float64,
-            DataType::Utf8,
-            DataType::Binary,
-            DataType::List(Box::new(Field::new("a", DataType::Binary, true))),
-        ];
-        let a = datatypes.into_iter().all(|x| new_empty_array(x).len() == 0);
-        assert!(a);
-    }
-
-    #[test]
-    fn test_clone() {
-        let datatypes = vec![
-            DataType::Int32,
-            DataType::Float64,
-            DataType::Utf8,
-            DataType::Binary,
-            DataType::List(Box::new(Field::new("a", DataType::Binary, true))),
-        ];
-        let a = datatypes
-            .into_iter()
-            .all(|x| clone(new_null_array(x.clone(), 10).as_ref()) == new_null_array(x, 10));
-        assert!(a);
-    }
-}
-
 // backward compatibility
 use std::sync::Arc;
 pub type ArrayRef = Arc<dyn Array>;
