@@ -20,6 +20,15 @@ pub use mutable::*;
 /// an array designed for highly performant operations on optionally nullable slots,
 /// backed by a physical type of a physical byte-width, such as `i32` or `f64`.
 /// The size of this struct is `O(1)` as all data is stored behind an [`std::sync::Arc`].
+/// # Example
+/// ```
+/// use arrow2::array::PrimitiveArray;
+/// # fn main() {
+/// let array = PrimitiveArray::<i32>::from([Some(1), None, Some(2)]);
+/// assert_eq!(array.value(0), 1);
+/// assert_eq!(array.values().as_slice(), &[1, 0, 2]);
+/// # }
+/// ```
 #[derive(Debug, Clone)]
 pub struct PrimitiveArray<T: NativeType> {
     data_type: DataType,
@@ -204,53 +213,3 @@ pub type UInt16Vec = MutablePrimitiveArray<u16>;
 pub type UInt32Vec = MutablePrimitiveArray<u32>;
 /// A type definition [`MutablePrimitiveArray`] for `u64`
 pub type UInt64Vec = MutablePrimitiveArray<u64>;
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use crate::array::Int32Array;
-    use std::iter::FromIterator;
-
-    #[test]
-    fn basics() {
-        let data = vec![Some(1), None, Some(10)];
-
-        let array = Int32Array::from_iter(data);
-
-        assert_eq!(array.value(0), 1);
-        assert_eq!(array.value(1), 0);
-        assert_eq!(array.value(2), 10);
-        assert_eq!(array.values().as_slice(), &[1, 0, 10]);
-        assert_eq!(
-            array.validity(),
-            &Some(Bitmap::from_u8_slice(&[0b00000101], 3))
-        );
-        assert!(array.is_valid(0));
-        assert!(!array.is_valid(1));
-        assert!(array.is_valid(2));
-
-        let array2 = PrimitiveArray::<i32>::from_data(
-            DataType::Int32,
-            array.values().clone(),
-            array.validity().clone(),
-        );
-        assert_eq!(array, array2);
-
-        let array = array.slice(1, 2);
-        assert_eq!(array.value(0), 0);
-        assert_eq!(array.value(1), 10);
-        assert_eq!(array.values().as_slice(), &[0, 10]);
-
-        unsafe {
-            assert_eq!(array.value_unchecked(0), 0);
-            assert_eq!(array.value_unchecked(1), 10);
-        }
-    }
-
-    #[test]
-    fn empty() {
-        let array = PrimitiveArray::<i32>::new_empty(DataType::Int32);
-        assert_eq!(array.values().len(), 0);
-        assert_eq!(array.validity(), &None);
-    }
-}
