@@ -13,17 +13,16 @@ mod mutable;
 pub use iterator::*;
 pub use mutable::*;
 
-/// A [`Utf8Array`] is arrow's equivalent of `Vec<Option<String>>`, i.e.
-/// an array designed for highly performant operations on optionally nullable strings.
-/// The size of this struct is `O(1)` as all data is stored behind an `Arc`.
+/// A [`Utf8Array`] is arrow's equivalent of an immutable `Vec<Option<String>>`.
+/// Cloning and slicing this struct is `O(1)`.
 /// # Example
 /// ```
-/// use std::iter::FromIterator;
 /// use arrow2::array::Utf8Array;
 /// # fn main() {
-/// let data = vec![Some("hello"), None, Some("hello2")];
-/// let array = Utf8Array::<i32>::from_iter(data);
-/// assert_eq!(array.value(0), "hello");
+/// let array = Utf8Array::<i32>::from([Some("hi"), None, Some("there")]);
+/// assert_eq!(array.value(0), "hi");
+/// assert_eq!(array.values().as_slice(), b"hithere".as_ref());
+/// assert_eq!(array.offsets().as_slice(), &[0, 2, 2, 2 + 5]);
 /// # }
 /// ```
 #[derive(Debug, Clone)]
@@ -200,52 +199,3 @@ unsafe impl<O: Offset> GenericBinaryArray<O> for Utf8Array<O> {
         self.offsets()
     }
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use std::iter::FromIterator;
-
-    #[test]
-    fn basics() {
-        let data = vec![Some("hello"), None, Some("hello2")];
-
-        let array = Utf8Array::<i32>::from_iter(data);
-
-        assert_eq!(array.value(0), "hello");
-        assert_eq!(array.value(1), "");
-        assert_eq!(array.value(2), "hello2");
-        assert_eq!(unsafe { array.value_unchecked(2) }, "hello2");
-        assert_eq!(array.values().as_slice(), b"hellohello2");
-        assert_eq!(array.offsets().as_slice(), &[0, 5, 5, 11]);
-        assert_eq!(
-            array.validity(),
-            &Some(Bitmap::from_u8_slice(&[0b00000101], 3))
-        );
-        assert!(array.is_valid(0));
-        assert!(!array.is_valid(1));
-        assert!(array.is_valid(2));
-
-        let array2 = Utf8Array::<i32>::from_data(
-            array.offsets().clone(),
-            array.values().clone(),
-            array.validity().clone(),
-        );
-        assert_eq!(array, array2);
-
-        let array = array.slice(1, 2);
-        assert_eq!(array.value(0), "");
-        assert_eq!(array.value(1), "hello2");
-        // note how this keeps everything: the offsets were sliced
-        assert_eq!(array.values().as_slice(), b"hellohello2");
-        assert_eq!(array.offsets().as_slice(), &[5, 5, 11]);
-    }
-
-    #[test]
-    fn empty() {
-        let array = Utf8Array::<i32>::new_empty();
-        assert_eq!(array.values().as_slice(), b"");
-        assert_eq!(array.offsets().as_slice(), &[0]);
-        assert_eq!(array.validity(), &None);
-    }
-}
