@@ -1,8 +1,9 @@
 //! Definition of basic div operations with primitive arrays
 use std::ops::Div;
 
-use num::{CheckedDiv, Zero};
+use num::{CheckedDiv, NumCast, Zero};
 
+use crate::datatypes::DataType;
 use crate::{
     array::{Array, PrimitiveArray},
     compute::{
@@ -12,6 +13,9 @@ use crate::{
     error::{ArrowError, Result},
     types::NativeType,
 };
+use strength_reduce::{
+    StrengthReducedU16, StrengthReducedU32, StrengthReducedU64, StrengthReducedU8,
+};
 
 /// Divides two primitive arrays with the same type.
 /// Panics if the divisor is zero of one pair of values overflows.
@@ -109,10 +113,72 @@ where
 /// ```
 pub fn div_scalar<T>(lhs: &PrimitiveArray<T>, rhs: &T) -> PrimitiveArray<T>
 where
-    T: NativeType + Div<Output = T>,
+    T: NativeType + Div<Output = T> + NumCast,
 {
     let rhs = *rhs;
-    unary(lhs, |a| a / rhs, lhs.data_type().clone())
+    match T::DATA_TYPE {
+        DataType::UInt64 => {
+            let lhs = lhs.as_any().downcast_ref::<PrimitiveArray<u64>>().unwrap();
+            let rhs = rhs.to_u64().unwrap();
+
+            let reduced_div = StrengthReducedU64::new(rhs);
+            // Safety: we just proved that `lhs` is `PrimitiveArray<u64>` which means that
+            // T = u64
+            unsafe {
+                std::mem::transmute::<PrimitiveArray<u64>, PrimitiveArray<T>>(unary(
+                    lhs,
+                    |a| a / reduced_div,
+                    lhs.data_type().clone(),
+                ))
+            }
+        }
+        DataType::UInt32 => {
+            let lhs = lhs.as_any().downcast_ref::<PrimitiveArray<u32>>().unwrap();
+            let rhs = rhs.to_u32().unwrap();
+
+            let reduced_div = StrengthReducedU32::new(rhs);
+            // Safety: we just proved that `lhs` is `PrimitiveArray<u32>` which means that
+            // T = u32
+            unsafe {
+                std::mem::transmute::<PrimitiveArray<u32>, PrimitiveArray<T>>(unary(
+                    lhs,
+                    |a| a / reduced_div,
+                    lhs.data_type().clone(),
+                ))
+            }
+        }
+        DataType::UInt16 => {
+            let lhs = lhs.as_any().downcast_ref::<PrimitiveArray<u16>>().unwrap();
+            let rhs = rhs.to_u16().unwrap();
+
+            let reduced_div = StrengthReducedU16::new(rhs);
+            // Safety: we just proved that `lhs` is `PrimitiveArray<u16>` which means that
+            // T = u16
+            unsafe {
+                std::mem::transmute::<PrimitiveArray<u16>, PrimitiveArray<T>>(unary(
+                    lhs,
+                    |a| a / reduced_div,
+                    lhs.data_type().clone(),
+                ))
+            }
+        }
+        DataType::UInt8 => {
+            let lhs = lhs.as_any().downcast_ref::<PrimitiveArray<u8>>().unwrap();
+            let rhs = rhs.to_u8().unwrap();
+
+            let reduced_div = StrengthReducedU8::new(rhs);
+            // Safety: we just proved that `lhs` is `PrimitiveArray<u8>` which means that
+            // T = u8
+            unsafe {
+                std::mem::transmute::<PrimitiveArray<u8>, PrimitiveArray<T>>(unary(
+                    lhs,
+                    |a| a / reduced_div,
+                    lhs.data_type().clone(),
+                ))
+            }
+        }
+        _ => unary(lhs, |a| a / rhs, lhs.data_type().clone()),
+    }
 }
 
 /// Checked division of a primitive array of type T by a scalar T. If the
@@ -141,7 +207,7 @@ where
 // Implementation of ArrayDiv trait for PrimitiveArrays with a scalar
 impl<T> ArrayDiv<T> for PrimitiveArray<T>
 where
-    T: NativeType + Div<Output = T> + NotI128,
+    T: NativeType + Div<Output = T> + NotI128 + NumCast,
 {
     type Output = Self;
 
@@ -226,6 +292,27 @@ mod tests {
         // Trait testing
         let result = a.div(&1i32).unwrap();
         assert_eq!(result, expected);
+
+        // check the strength reduced branches
+        let a = UInt64Array::from(&[None, Some(6), None, Some(6)]);
+        let result = div_scalar(&a, &1u64);
+        let expected = UInt64Array::from(&[None, Some(6), None, Some(6)]);
+        assert_eq!(result, expected);
+
+        let a = UInt32Array::from(&[None, Some(6), None, Some(6)]);
+        let result = div_scalar(&a, &1u32);
+        let expected = UInt32Array::from(&[None, Some(6), None, Some(6)]);
+        assert_eq!(result, expected);
+
+        let a = UInt16Array::from(&[None, Some(6), None, Some(6)]);
+        let result = div_scalar(&a, &1u16);
+        let expected = UInt16Array::from(&[None, Some(6), None, Some(6)]);
+        assert_eq!(result, expected);
+
+        let a = UInt8Array::from(&[None, Some(6), None, Some(6)]);
+        let result = div_scalar(&a, &1u8);
+        let expected = UInt8Array::from(&[None, Some(6), None, Some(6)]);
+        assert_eq!(result, expected);
     }
 
     #[test]
