@@ -42,11 +42,11 @@ use super::{adjusted_precision_scale, max_value, number_digits};
 /// use arrow2::array::PrimitiveArray;
 /// use arrow2::datatypes::DataType;
 ///
-/// let a = PrimitiveArray::from(&vec![Some(1i128), Some(1i128), None, Some(2i128)]).to(DataType::Decimal(5, 2));
-/// let b = PrimitiveArray::from(&vec![Some(1i128), Some(2i128), None, Some(2i128)]).to(DataType::Decimal(5, 2));
+/// let a = PrimitiveArray::from([Some(1i128), Some(1i128), None, Some(2i128)]).to(DataType::Decimal(5, 2));
+/// let b = PrimitiveArray::from([Some(1i128), Some(2i128), None, Some(2i128)]).to(DataType::Decimal(5, 2));
 ///
 /// let result = sub(&a, &b).unwrap();
-/// let expected = PrimitiveArray::from(&vec![Some(0i128), Some(-1i128), None, Some(0i128)]).to(DataType::Decimal(5, 2));
+/// let expected = PrimitiveArray::from([Some(0i128), Some(-1i128), None, Some(0i128)]).to(DataType::Decimal(5, 2));
 ///
 /// assert_eq!(result, expected);
 /// ```
@@ -95,11 +95,11 @@ pub fn sub(lhs: &PrimitiveArray<i128>, rhs: &PrimitiveArray<i128>) -> Result<Pri
 /// use arrow2::array::PrimitiveArray;
 /// use arrow2::datatypes::DataType;
 ///
-/// let a = PrimitiveArray::from(&vec![Some(-99000i128), Some(11100i128), None, Some(22200i128)]).to(DataType::Decimal(5, 2));
-/// let b = PrimitiveArray::from(&vec![Some(01000i128), Some(22200i128), None, Some(11100i128)]).to(DataType::Decimal(5, 2));
+/// let a = PrimitiveArray::from([Some(-99000i128), Some(11100i128), None, Some(22200i128)]).to(DataType::Decimal(5, 2));
+/// let b = PrimitiveArray::from([Some(01000i128), Some(22200i128), None, Some(11100i128)]).to(DataType::Decimal(5, 2));
 ///
 /// let result = saturating_sub(&a, &b).unwrap();
-/// let expected = PrimitiveArray::from(&vec![Some(-99999i128), Some(-11100i128), None, Some(11100i128)]).to(DataType::Decimal(5, 2));
+/// let expected = PrimitiveArray::from([Some(-99999i128), Some(-11100i128), None, Some(11100i128)]).to(DataType::Decimal(5, 2));
 ///
 /// assert_eq!(result, expected);
 /// ```
@@ -181,11 +181,11 @@ impl ArraySaturatingSub<PrimitiveArray<i128>> for PrimitiveArray<i128> {
 /// use arrow2::array::PrimitiveArray;
 /// use arrow2::datatypes::DataType;
 ///
-/// let a = PrimitiveArray::from(&vec![Some(-99000i128), Some(11100i128), None, Some(22200i128)]).to(DataType::Decimal(5, 2));
-/// let b = PrimitiveArray::from(&vec![Some(01000i128), Some(22200i128), None, Some(11100i128)]).to(DataType::Decimal(5, 2));
+/// let a = PrimitiveArray::from([Some(-99000i128), Some(11100i128), None, Some(22200i128)]).to(DataType::Decimal(5, 2));
+/// let b = PrimitiveArray::from([Some(01000i128), Some(22200i128), None, Some(11100i128)]).to(DataType::Decimal(5, 2));
 ///
 /// let result = checked_sub(&a, &b).unwrap();
-/// let expected = PrimitiveArray::from(&vec![None, Some(-11100i128), None, Some(11100i128)]).to(DataType::Decimal(5, 2));
+/// let expected = PrimitiveArray::from([None, Some(-11100i128), None, Some(11100i128)]).to(DataType::Decimal(5, 2));
 ///
 /// assert_eq!(result, expected);
 /// ```
@@ -240,10 +240,10 @@ pub fn checked_sub(
 /// use arrow2::array::PrimitiveArray;
 /// use arrow2::datatypes::DataType;
 ///
-/// let a = PrimitiveArray::from(&vec![Some(99_9999i128)]).to(DataType::Decimal(6, 4));
-/// let b = PrimitiveArray::from(&vec![Some(-00_0001i128)]).to(DataType::Decimal(6, 4));
+/// let a = PrimitiveArray::from([Some(99_9999i128)]).to(DataType::Decimal(6, 4));
+/// let b = PrimitiveArray::from([Some(-00_0001i128)]).to(DataType::Decimal(6, 4));
 /// let result = adaptive_sub(&a, &b).unwrap();
-/// let expected = PrimitiveArray::from(&vec![Some(100_0000i128)]).to(DataType::Decimal(7, 4));
+/// let expected = PrimitiveArray::from([Some(100_0000i128)]).to(DataType::Decimal(7, 4));
 ///
 /// assert_eq!(result, expected);
 /// ```
@@ -314,7 +314,7 @@ mod tests {
 
     #[test]
     fn test_subtract_normal() {
-        let a = PrimitiveArray::from(&vec![
+        let a = PrimitiveArray::from([
             Some(11111i128),
             Some(22200i128),
             None,
@@ -322,7 +322,7 @@ mod tests {
         ])
         .to(DataType::Decimal(5, 2));
 
-        let b = PrimitiveArray::from(&vec![
+        let b = PrimitiveArray::from([
             Some(22222i128),
             Some(11100i128),
             None,
@@ -331,7 +331,7 @@ mod tests {
         .to(DataType::Decimal(5, 2));
 
         let result = sub(&a, &b).unwrap();
-        let expected = PrimitiveArray::from(&vec![
+        let expected = PrimitiveArray::from([
             Some(-11111i128),
             Some(11100i128),
             None,
@@ -348,8 +348,8 @@ mod tests {
 
     #[test]
     fn test_subtract_decimal_wrong_precision() {
-        let a = PrimitiveArray::from(&vec![None]).to(DataType::Decimal(5, 2));
-        let b = PrimitiveArray::from(&vec![None]).to(DataType::Decimal(6, 2));
+        let a = PrimitiveArray::from([None]).to(DataType::Decimal(5, 2));
+        let b = PrimitiveArray::from([None]).to(DataType::Decimal(6, 2));
         let result = sub(&a, &b);
 
         if result.is_ok() {
@@ -360,14 +360,14 @@ mod tests {
     #[test]
     #[should_panic(expected = "Overflow in subtract presented for precision 5")]
     fn test_subtract_panic() {
-        let a = PrimitiveArray::from(&vec![Some(-99999i128)]).to(DataType::Decimal(5, 2));
-        let b = PrimitiveArray::from(&vec![Some(1i128)]).to(DataType::Decimal(5, 2));
+        let a = PrimitiveArray::from([Some(-99999i128)]).to(DataType::Decimal(5, 2));
+        let b = PrimitiveArray::from([Some(1i128)]).to(DataType::Decimal(5, 2));
         let _ = sub(&a, &b);
     }
 
     #[test]
     fn test_subtract_saturating() {
-        let a = PrimitiveArray::from(&vec![
+        let a = PrimitiveArray::from([
             Some(11111i128),
             Some(22200i128),
             None,
@@ -375,7 +375,7 @@ mod tests {
         ])
         .to(DataType::Decimal(5, 2));
 
-        let b = PrimitiveArray::from(&vec![
+        let b = PrimitiveArray::from([
             Some(22222i128),
             Some(11100i128),
             None,
@@ -384,7 +384,7 @@ mod tests {
         .to(DataType::Decimal(5, 2));
 
         let result = saturating_sub(&a, &b).unwrap();
-        let expected = PrimitiveArray::from(&vec![
+        let expected = PrimitiveArray::from([
             Some(-11111i128),
             Some(11100i128),
             None,
@@ -401,14 +401,14 @@ mod tests {
 
     #[test]
     fn test_subtract_saturating_overflow() {
-        let a = PrimitiveArray::from(&vec![
+        let a = PrimitiveArray::from([
             Some(-99999i128),
             Some(-99999i128),
             Some(-99999i128),
             Some(99999i128),
         ])
         .to(DataType::Decimal(5, 2));
-        let b = PrimitiveArray::from(&vec![
+        let b = PrimitiveArray::from([
             Some(00001i128),
             Some(00100i128),
             Some(10000i128),
@@ -418,7 +418,7 @@ mod tests {
 
         let result = saturating_sub(&a, &b).unwrap();
 
-        let expected = PrimitiveArray::from(&vec![
+        let expected = PrimitiveArray::from([
             Some(-99999i128),
             Some(-99999i128),
             Some(-99999i128),
@@ -435,7 +435,7 @@ mod tests {
 
     #[test]
     fn test_subtract_checked() {
-        let a = PrimitiveArray::from(&vec![
+        let a = PrimitiveArray::from([
             Some(11111i128),
             Some(22200i128),
             None,
@@ -443,7 +443,7 @@ mod tests {
         ])
         .to(DataType::Decimal(5, 2));
 
-        let b = PrimitiveArray::from(&vec![
+        let b = PrimitiveArray::from([
             Some(22222i128),
             Some(11100i128),
             None,
@@ -452,7 +452,7 @@ mod tests {
         .to(DataType::Decimal(5, 2));
 
         let result = checked_sub(&a, &b).unwrap();
-        let expected = PrimitiveArray::from(&vec![
+        let expected = PrimitiveArray::from([
             Some(-11111i128),
             Some(11100i128),
             None,
@@ -470,10 +470,10 @@ mod tests {
     #[test]
     fn test_subtract_checked_overflow() {
         let a =
-            PrimitiveArray::from(&vec![Some(4i128), Some(-99999i128)]).to(DataType::Decimal(5, 2));
-        let b = PrimitiveArray::from(&vec![Some(2i128), Some(1i128)]).to(DataType::Decimal(5, 2));
+            PrimitiveArray::from([Some(4i128), Some(-99999i128)]).to(DataType::Decimal(5, 2));
+        let b = PrimitiveArray::from([Some(2i128), Some(1i128)]).to(DataType::Decimal(5, 2));
         let result = checked_sub(&a, &b).unwrap();
-        let expected = PrimitiveArray::from(&vec![Some(2i128), None]).to(DataType::Decimal(5, 2));
+        let expected = PrimitiveArray::from([Some(2i128), None]).to(DataType::Decimal(5, 2));
         assert_eq!(result, expected);
     }
 
@@ -483,12 +483,12 @@ mod tests {
         //  11111.11   -> 7, 2
         // ------------------
         // -11099.9989 -> 9, 4
-        let a = PrimitiveArray::from(&vec![Some(11_1111i128)]).to(DataType::Decimal(6, 4));
-        let b = PrimitiveArray::from(&vec![Some(11111_11i128)]).to(DataType::Decimal(7, 2));
+        let a = PrimitiveArray::from([Some(11_1111i128)]).to(DataType::Decimal(6, 4));
+        let b = PrimitiveArray::from([Some(11111_11i128)]).to(DataType::Decimal(7, 2));
         let result = adaptive_sub(&a, &b).unwrap();
 
         let expected =
-            PrimitiveArray::from(&vec![Some(-11099_9989i128)]).to(DataType::Decimal(9, 4));
+            PrimitiveArray::from([Some(-11099_9989i128)]).to(DataType::Decimal(9, 4));
 
         assert_eq!(result, expected);
         assert_eq!(result.data_type(), &DataType::Decimal(9, 4));
@@ -497,12 +497,12 @@ mod tests {
         //     0.1111 -> 5, 4
         // -----------------
         // 11110.8889 -> 9, 4
-        let a = PrimitiveArray::from(&vec![Some(11111_0i128)]).to(DataType::Decimal(6, 1));
-        let b = PrimitiveArray::from(&vec![Some(1111i128)]).to(DataType::Decimal(5, 4));
+        let a = PrimitiveArray::from([Some(11111_0i128)]).to(DataType::Decimal(6, 1));
+        let b = PrimitiveArray::from([Some(1111i128)]).to(DataType::Decimal(5, 4));
         let result = adaptive_sub(&a, &b).unwrap();
 
         let expected =
-            PrimitiveArray::from(&vec![Some(11110_8889i128)]).to(DataType::Decimal(9, 4));
+            PrimitiveArray::from([Some(11110_8889i128)]).to(DataType::Decimal(9, 4));
 
         assert_eq!(result, expected);
         assert_eq!(result.data_type(), &DataType::Decimal(9, 4));
@@ -511,12 +511,12 @@ mod tests {
         //  11111.111  -> 8, 3
         // -----------------
         // -00000.001  -> 8, 3
-        let a = PrimitiveArray::from(&vec![Some(11111_11i128)]).to(DataType::Decimal(7, 2));
-        let b = PrimitiveArray::from(&vec![Some(11111_111i128)]).to(DataType::Decimal(8, 3));
+        let a = PrimitiveArray::from([Some(11111_11i128)]).to(DataType::Decimal(7, 2));
+        let b = PrimitiveArray::from([Some(11111_111i128)]).to(DataType::Decimal(8, 3));
         let result = adaptive_sub(&a, &b).unwrap();
 
         let expected =
-            PrimitiveArray::from(&vec![Some(-00000_001i128)]).to(DataType::Decimal(8, 3));
+            PrimitiveArray::from([Some(-00000_001i128)]).to(DataType::Decimal(8, 3));
 
         assert_eq!(result, expected);
         assert_eq!(result.data_type(), &DataType::Decimal(8, 3));
@@ -525,11 +525,11 @@ mod tests {
         // -00.0001 -> 6, 4
         // -----------------
         // 100.0000 -> 7, 4
-        let a = PrimitiveArray::from(&vec![Some(99_9999i128)]).to(DataType::Decimal(6, 4));
-        let b = PrimitiveArray::from(&vec![Some(-00_0001i128)]).to(DataType::Decimal(6, 4));
+        let a = PrimitiveArray::from([Some(99_9999i128)]).to(DataType::Decimal(6, 4));
+        let b = PrimitiveArray::from([Some(-00_0001i128)]).to(DataType::Decimal(6, 4));
         let result = adaptive_sub(&a, &b).unwrap();
 
-        let expected = PrimitiveArray::from(&vec![Some(100_0000i128)]).to(DataType::Decimal(7, 4));
+        let expected = PrimitiveArray::from([Some(100_0000i128)]).to(DataType::Decimal(7, 4));
 
         assert_eq!(result, expected);
         assert_eq!(result.data_type(), &DataType::Decimal(7, 4));
