@@ -6,7 +6,7 @@ use crate::datatypes::*;
 use crate::error::{ArrowError, Result};
 use crate::{
     array::*,
-    types::{days_ms, Index, NativeType},
+    types::{Index, NativeType},
 };
 
 use crate::buffer::MutableBuffer;
@@ -63,9 +63,6 @@ pub fn sort(
         DataType::UInt64 => dyn_sort!(u64, values, ord::total_cmp, options, limit),
         DataType::Float32 => dyn_sort!(f32, values, ord::total_cmp_f32, options, limit),
         DataType::Float64 => dyn_sort!(f64, values, ord::total_cmp_f64, options, limit),
-        DataType::Interval(IntervalUnit::DayTime) => {
-            dyn_sort!(days_ms, values, ord::total_cmp, options, limit)
-        }
         _ => {
             let indices = sort_to_indices::<u64>(values, options, limit)?;
             take::take(values, &indices)
@@ -134,9 +131,6 @@ pub fn sort_to_indices<I: Index>(
         DataType::UInt64 => dyn_sort_indices!(I, u64, values, ord::total_cmp, options, limit),
         DataType::Float32 => dyn_sort_indices!(I, f32, values, ord::total_cmp_f32, options, limit),
         DataType::Float64 => dyn_sort_indices!(I, f64, values, ord::total_cmp_f64, options, limit),
-        DataType::Interval(IntervalUnit::DayTime) => {
-            dyn_sort_indices!(I, days_ms, values, ord::total_cmp, options, limit)
-        }
         DataType::Utf8 => Ok(utf8::indices_sorted_unstable_by::<I, i32>(
             values.as_any().downcast_ref().unwrap(),
             options,
@@ -199,8 +193,18 @@ pub fn sort_to_indices<I: Index>(
             }
         }
         DataType::Dictionary(key_type, value_type) => match value_type.as_ref() {
-            DataType::Utf8 => sort_dict::<I, i32>(values, key_type.as_ref(), options, limit),
-            DataType::LargeUtf8 => sort_dict::<I, i64>(values, key_type.as_ref(), options, limit),
+            DataType::Utf8 => Ok(sort_dict::<I, i32>(
+                values,
+                key_type.as_ref(),
+                options,
+                limit,
+            )),
+            DataType::LargeUtf8 => Ok(sort_dict::<I, i64>(
+                values,
+                key_type.as_ref(),
+                options,
+                limit,
+            )),
             t => Err(ArrowError::NotYetImplemented(format!(
                 "Sort not supported for dictionary type with keys {:?}",
                 t
@@ -218,53 +222,14 @@ fn sort_dict<I: Index, O: Offset>(
     key_type: &DataType,
     options: &SortOptions,
     limit: Option<usize>,
-) -> Result<PrimitiveArray<I>> {
-    match key_type {
-        DataType::Int8 => Ok(utf8::indices_sorted_unstable_by_dictionary::<I, i8, O>(
+) -> PrimitiveArray<I> {
+    with_match_dictionary_key_type!(key_type, |$T| {
+        utf8::indices_sorted_unstable_by_dictionary::<I, $T, O>(
             values.as_any().downcast_ref().unwrap(),
             options,
             limit,
-        )),
-        DataType::Int16 => Ok(utf8::indices_sorted_unstable_by_dictionary::<I, i16, O>(
-            values.as_any().downcast_ref().unwrap(),
-            options,
-            limit,
-        )),
-        DataType::Int32 => Ok(utf8::indices_sorted_unstable_by_dictionary::<I, i32, O>(
-            values.as_any().downcast_ref().unwrap(),
-            options,
-            limit,
-        )),
-        DataType::Int64 => Ok(utf8::indices_sorted_unstable_by_dictionary::<I, i64, O>(
-            values.as_any().downcast_ref().unwrap(),
-            options,
-            limit,
-        )),
-        DataType::UInt8 => Ok(utf8::indices_sorted_unstable_by_dictionary::<I, u8, O>(
-            values.as_any().downcast_ref().unwrap(),
-            options,
-            limit,
-        )),
-        DataType::UInt16 => Ok(utf8::indices_sorted_unstable_by_dictionary::<I, u16, O>(
-            values.as_any().downcast_ref().unwrap(),
-            options,
-            limit,
-        )),
-        DataType::UInt32 => Ok(utf8::indices_sorted_unstable_by_dictionary::<I, u32, O>(
-            values.as_any().downcast_ref().unwrap(),
-            options,
-            limit,
-        )),
-        DataType::UInt64 => Ok(utf8::indices_sorted_unstable_by_dictionary::<I, u64, O>(
-            values.as_any().downcast_ref().unwrap(),
-            options,
-            limit,
-        )),
-        t => Err(ArrowError::NotYetImplemented(format!(
-            "Sort not supported for dictionary key type {:?}",
-            t
-        ))),
-    }
+        )
+    })
 }
 
 /// Checks if an array of type `datatype` can be sorted
