@@ -197,85 +197,3 @@ where
 
     RecordBatch::try_new(schema, columns)
 }
-
-#[cfg(test)]
-mod tests {
-    use super::super::*;
-    use super::*;
-
-    fn test(input: &str, data_type: DataType) -> Result<Arc<dyn Array>> {
-        let reader = std::io::Cursor::new(input);
-        let mut reader = ReaderBuilder::new().has_headers(false).from_reader(reader);
-
-        let mut rows = vec![ByteRecord::default(); 10];
-        let rows_read = read_rows(&mut reader, 0, &mut rows)?;
-        deserialize_column(&rows[..rows_read], 0, data_type, 0)
-    }
-
-    #[test]
-    fn int32() -> Result<()> {
-        let result = test("1,\n,\n3,", DataType::Int32)?;
-        let expected = Int32Array::from(&[Some(1), None, Some(3)]);
-        assert_eq!(expected, result.as_ref());
-        Ok(())
-    }
-
-    #[test]
-    fn date32() -> Result<()> {
-        let result = test("1970-01-01,\n2020-03-15,\n1945-05-08,\n", DataType::Date32)?;
-        let expected = Int32Array::from(&[Some(0), Some(18336), Some(-9004)]).to(DataType::Date32);
-        assert_eq!(expected, result.as_ref());
-        Ok(())
-    }
-
-    #[test]
-    fn date64() -> Result<()> {
-        let input = "1970-01-01T00:00:00,\n \
-            2018-11-13T17:11:10,\n \
-            2018-11-13T17:11:10.011,\n \
-            1900-02-28T12:34:56,\n";
-
-        let result = test(input, DataType::Date64)?;
-        let expected = Int64Array::from(&[
-            Some(0),
-            Some(1542129070000),
-            Some(1542129070011),
-            Some(-2203932304000),
-        ])
-        .to(DataType::Date64);
-        assert_eq!(expected, result.as_ref());
-        Ok(())
-    }
-
-    #[test]
-    fn boolean() -> Result<()> {
-        let input = vec!["true", "True", "False", "F", "t"];
-        let input = input.join("\n");
-
-        let expected = BooleanArray::from(&[Some(true), Some(true), Some(false), None, None]);
-
-        let result = test(&input, DataType::Boolean)?;
-
-        assert_eq!(expected, result.as_ref());
-        Ok(())
-    }
-
-    #[test]
-    fn float32() -> Result<()> {
-        let input = vec!["12.34", "12", "0.0", "inf", "-inf", "dd"];
-        let input = input.join("\n");
-
-        let expected = Float32Array::from(&[
-            Some(12.34),
-            Some(12.0),
-            Some(0.0),
-            Some(f32::INFINITY),
-            Some(f32::NEG_INFINITY),
-            None,
-        ]);
-
-        let result = test(&input, DataType::Float32)?;
-        assert_eq!(expected, result.as_ref());
-        Ok(())
-    }
-}
