@@ -19,6 +19,7 @@
 
 use crate::datatypes::{DataType, Field, IntervalUnit, Schema, TimeUnit};
 use crate::endianess::is_native_little_endian;
+use crate::io::ipc::convert::ipc::UnionMode;
 
 mod ipc {
     pub use super::super::gen::File::*;
@@ -276,6 +277,22 @@ pub(crate) fn get_data_type(field: ipc::Field, may_be_dictionary: bool) -> DataT
             let fsb = field.type_as_decimal().unwrap();
             DataType::Decimal(fsb.precision() as usize, fsb.scale() as usize)
         }
+        ipc::Type::Union => {
+            let type_ = field.type_as_union().unwrap();
+
+            let is_sparse = type_.mode() == UnionMode::Sparse;
+
+            let ids = type_.typeIds().map(|x| x.iter().collect());
+
+            let fields = if let Some(children) = field.children() {
+                (0..children.len())
+                    .map(|i| children.get(i).into())
+                    .collect()
+            } else {
+                vec![]
+            };
+            DataType::Union(fields, ids, is_sparse)
+        }
         t => unimplemented!("Type {:?} not supported", t),
     }
 }
@@ -346,56 +363,71 @@ pub(crate) fn build_field<'a>(
     field_builder.finish()
 }
 
+fn type_to_field_type(data_type: &DataType) -> ipc::Type {
+    match data_type {
+        Null => ipc::Type::Null,
+        Boolean => ipc::Type::Bool,
+        UInt8 | UInt16 | UInt32 | UInt64 | Int8 | Int16 | Int32 | Int64 => ipc::Type::Int,
+        Float16 | Float32 | Float64 => ipc::Type::FloatingPoint,
+        Decimal(_, _) => ipc::Type::Decimal,
+        Binary => ipc::Type::Binary,
+        LargeBinary => ipc::Type::LargeBinary,
+        Utf8 => ipc::Type::Utf8,
+        LargeUtf8 => ipc::Type::LargeUtf8,
+        FixedSizeBinary(_) => ipc::Type::FixedSizeBinary,
+        Date32 | Date64 => ipc::Type::Date,
+        Duration(_) => ipc::Type::Duration,
+        Time32(_) | Time64(_) => ipc::Type::Time,
+        Timestamp(_, _) => ipc::Type::Timestamp,
+        Interval(_) => ipc::Type::Interval,
+        List(_) => ipc::Type::List,
+        LargeList(_) => ipc::Type::LargeList,
+        FixedSizeList(_, _) => ipc::Type::FixedSizeList,
+        Union(_, _, _) => ipc::Type::Union,
+        Struct(_) => ipc::Type::Struct_,
+        Dictionary(_, v) => type_to_field_type(v),
+    }
+}
+
 /// Get the IPC type of a data type
 pub(crate) fn get_fb_field_type<'a>(
     data_type: &DataType,
     is_nullable: bool,
     fbb: &mut FlatBufferBuilder<'a>,
 ) -> FbFieldType<'a> {
+    let type_type = type_to_field_type(data_type);
+
     // some IPC implementations expect an empty list for child data, instead of a null value.
     // An empty field list is thus returned for primitive types
     let empty_fields: Vec<WIPOffset<ipc::Field>> = vec![];
     match data_type {
         Null => FbFieldType {
-            type_type: ipc::Type::Null,
+            type_type,
             type_: ipc::NullBuilder::new(fbb).finish().as_union_value(),
             children: Some(fbb.create_vector(&empty_fields[..])),
         },
         Boolean => FbFieldType {
-            type_type: ipc::Type::Bool,
+            type_type,
             type_: ipc::BoolBuilder::new(fbb).finish().as_union_value(),
             children: Some(fbb.create_vector(&empty_fields[..])),
         },
-        UInt8 | UInt16 | UInt32 | UInt64 => {
+        Int8 | Int16 | Int32 | Int64 | UInt8 | UInt16 | UInt32 | UInt64 => {
             let children = fbb.create_vector(&empty_fields[..]);
             let mut builder = ipc::IntBuilder::new(fbb);
-            builder.add_is_signed(false);
-            match data_type {
-                UInt8 => builder.add_bitWidth(8),
-                UInt16 => builder.add_bitWidth(16),
-                UInt32 => builder.add_bitWidth(32),
-                UInt64 => builder.add_bitWidth(64),
-                _ => {}
-            };
-            FbFieldType {
-                type_type: ipc::Type::Int,
-                type_: builder.finish().as_union_value(),
-                children: Some(children),
+            if matches!(data_type, UInt8 | UInt16 | UInt32 | UInt64) {
+                builder.add_is_signed(false);
+            } else {
+                builder.add_is_signed(true);
             }
-        }
-        Int8 | Int16 | Int32 | Int64 => {
-            let children = fbb.create_vector(&empty_fields[..]);
-            let mut builder = ipc::IntBuilder::new(fbb);
-            builder.add_is_signed(true);
             match data_type {
-                Int8 => builder.add_bitWidth(8),
-                Int16 => builder.add_bitWidth(16),
-                Int32 => builder.add_bitWidth(32),
-                Int64 => builder.add_bitWidth(64),
+                Int8 | UInt8 => builder.add_bitWidth(8),
+                Int16 | UInt16 => builder.add_bitWidth(16),
+                Int32 | UInt32 => builder.add_bitWidth(32),
+                Int64 | UInt64 => builder.add_bitWidth(64),
                 _ => {}
             };
             FbFieldType {
-                type_type: ipc::Type::Int,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(children),
             }
@@ -410,28 +442,28 @@ pub(crate) fn get_fb_field_type<'a>(
                 _ => {}
             };
             FbFieldType {
-                type_type: ipc::Type::FloatingPoint,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(children),
             }
         }
         Binary => FbFieldType {
-            type_type: ipc::Type::Binary,
+            type_type,
             type_: ipc::BinaryBuilder::new(fbb).finish().as_union_value(),
             children: Some(fbb.create_vector(&empty_fields[..])),
         },
         LargeBinary => FbFieldType {
-            type_type: ipc::Type::LargeBinary,
+            type_type,
             type_: ipc::LargeBinaryBuilder::new(fbb).finish().as_union_value(),
             children: Some(fbb.create_vector(&empty_fields[..])),
         },
         Utf8 => FbFieldType {
-            type_type: ipc::Type::Utf8,
+            type_type,
             type_: ipc::Utf8Builder::new(fbb).finish().as_union_value(),
             children: Some(fbb.create_vector(&empty_fields[..])),
         },
         LargeUtf8 => FbFieldType {
-            type_type: ipc::Type::LargeUtf8,
+            type_type,
             type_: ipc::LargeUtf8Builder::new(fbb).finish().as_union_value(),
             children: Some(fbb.create_vector(&empty_fields[..])),
         },
@@ -439,7 +471,7 @@ pub(crate) fn get_fb_field_type<'a>(
             let mut builder = ipc::FixedSizeBinaryBuilder::new(fbb);
             builder.add_byteWidth(*len as i32);
             FbFieldType {
-                type_type: ipc::Type::FixedSizeBinary,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(fbb.create_vector(&empty_fields[..])),
             }
@@ -448,7 +480,7 @@ pub(crate) fn get_fb_field_type<'a>(
             let mut builder = ipc::DateBuilder::new(fbb);
             builder.add_unit(ipc::DateUnit::DAY);
             FbFieldType {
-                type_type: ipc::Type::Date,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(fbb.create_vector(&empty_fields[..])),
             }
@@ -457,7 +489,7 @@ pub(crate) fn get_fb_field_type<'a>(
             let mut builder = ipc::DateBuilder::new(fbb);
             builder.add_unit(ipc::DateUnit::MILLISECOND);
             FbFieldType {
-                type_type: ipc::Type::Date,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(fbb.create_vector(&empty_fields[..])),
             }
@@ -483,7 +515,7 @@ pub(crate) fn get_fb_field_type<'a>(
                 }
             }
             FbFieldType {
-                type_type: ipc::Type::Time,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(fbb.create_vector(&empty_fields[..])),
             }
@@ -503,7 +535,7 @@ pub(crate) fn get_fb_field_type<'a>(
                 builder.add_timezone(tz_str);
             }
             FbFieldType {
-                type_type: ipc::Type::Timestamp,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(fbb.create_vector(&empty_fields[..])),
             }
@@ -516,7 +548,7 @@ pub(crate) fn get_fb_field_type<'a>(
             };
             builder.add_unit(interval_unit);
             FbFieldType {
-                type_type: ipc::Type::Interval,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(fbb.create_vector(&empty_fields[..])),
             }
@@ -531,7 +563,7 @@ pub(crate) fn get_fb_field_type<'a>(
             };
             builder.add_unit(time_unit);
             FbFieldType {
-                type_type: ipc::Type::Duration,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(fbb.create_vector(&empty_fields[..])),
             }
@@ -539,7 +571,7 @@ pub(crate) fn get_fb_field_type<'a>(
         List(ref list_type) => {
             let child = build_field(fbb, list_type);
             FbFieldType {
-                type_type: ipc::Type::List,
+                type_type,
                 type_: ipc::ListBuilder::new(fbb).finish().as_union_value(),
                 children: Some(fbb.create_vector(&[child])),
             }
@@ -547,7 +579,7 @@ pub(crate) fn get_fb_field_type<'a>(
         LargeList(ref list_type) => {
             let child = build_field(fbb, list_type);
             FbFieldType {
-                type_type: ipc::Type::LargeList,
+                type_type,
                 type_: ipc::LargeListBuilder::new(fbb).finish().as_union_value(),
                 children: Some(fbb.create_vector(&[child])),
             }
@@ -557,7 +589,7 @@ pub(crate) fn get_fb_field_type<'a>(
             let mut builder = ipc::FixedSizeListBuilder::new(fbb);
             builder.add_listSize(*len as i32);
             FbFieldType {
-                type_type: ipc::Type::FixedSizeList,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(fbb.create_vector(&[child])),
             }
@@ -582,7 +614,7 @@ pub(crate) fn get_fb_field_type<'a>(
                 ));
             }
             FbFieldType {
-                type_type: ipc::Type::Struct_,
+                type_type,
                 type_: ipc::Struct_Builder::new(fbb).finish().as_union_value(),
                 children: Some(fbb.create_vector(&children[..])),
             }
@@ -599,12 +631,32 @@ pub(crate) fn get_fb_field_type<'a>(
             builder.add_scale(*scale as i32);
             builder.add_bitWidth(128);
             FbFieldType {
-                type_type: ipc::Type::Decimal,
+                type_type,
                 type_: builder.finish().as_union_value(),
                 children: Some(fbb.create_vector(&empty_fields[..])),
             }
         }
-        t => unimplemented!("Type {:?} not supported", t),
+        Union(fields, ids, is_sparse) => {
+            let children: Vec<_> = fields.iter().map(|field| build_field(fbb, field)).collect();
+
+            let ids = ids.as_ref().map(|ids| fbb.create_vector(ids));
+
+            let mut builder = ipc::UnionBuilder::new(fbb);
+            builder.add_mode(if *is_sparse {
+                UnionMode::Sparse
+            } else {
+                UnionMode::Dense
+            });
+
+            if let Some(ids) = ids {
+                builder.add_typeIds(ids);
+            }
+            FbFieldType {
+                type_type,
+                type_: builder.finish().as_union_value(),
+                children: Some(fbb.create_vector(&children)),
+            }
+        }
     }
 }
 
