@@ -16,10 +16,7 @@
 // under the License.
 
 use crate::{
-    array::{
-        Array, BinaryArray, BooleanArray, DictionaryArray, DictionaryKey, FixedSizeBinaryArray,
-        FixedSizeListArray, ListArray, Offset, PrimitiveArray, StructArray, Utf8Array,
-    },
+    array::*,
     bitmap::Bitmap,
     datatypes::{DataType, IntervalUnit},
     endianess::is_native_little_endian,
@@ -236,6 +233,33 @@ pub fn write_struct(
     });
 }
 
+pub fn write_union(
+    array: &dyn Array,
+    buffers: &mut Vec<Schema::Buffer>,
+    arrow_data: &mut Vec<u8>,
+    nodes: &mut Vec<Message::FieldNode>,
+    offset: &mut i64,
+    is_little_endian: bool,
+) {
+    let array = array.as_any().downcast_ref::<UnionArray>().unwrap();
+
+    write_buffer(array.types(), buffers, arrow_data, offset, is_little_endian);
+
+    if let Some(offsets) = array.offsets() {
+        write_buffer(offsets, buffers, arrow_data, offset, is_little_endian);
+    }
+    array.fields().iter().for_each(|array| {
+        write(
+            array.as_ref(),
+            buffers,
+            arrow_data,
+            nodes,
+            offset,
+            is_little_endian,
+        )
+    });
+}
+
 fn write_fixed_size_list(
     array: &dyn Array,
     buffers: &mut Vec<Schema::Buffer>,
@@ -293,81 +317,19 @@ pub fn write_dictionary(
     write_keys: bool,
 ) -> usize {
     match array.data_type() {
-        DataType::Dictionary(key_type, _) => match key_type.as_ref() {
-            DataType::Int8 => _write_dictionary::<i8>(
-                array,
-                buffers,
-                arrow_data,
-                nodes,
-                offset,
-                is_little_endian,
-                write_keys,
-            ),
-            DataType::Int16 => _write_dictionary::<i16>(
-                array,
-                buffers,
-                arrow_data,
-                nodes,
-                offset,
-                is_little_endian,
-                write_keys,
-            ),
-            DataType::Int32 => _write_dictionary::<i32>(
-                array,
-                buffers,
-                arrow_data,
-                nodes,
-                offset,
-                is_little_endian,
-                write_keys,
-            ),
-            DataType::Int64 => _write_dictionary::<i64>(
-                array,
-                buffers,
-                arrow_data,
-                nodes,
-                offset,
-                is_little_endian,
-                write_keys,
-            ),
-            DataType::UInt8 => _write_dictionary::<u8>(
-                array,
-                buffers,
-                arrow_data,
-                nodes,
-                offset,
-                is_little_endian,
-                write_keys,
-            ),
-            DataType::UInt16 => _write_dictionary::<u16>(
-                array,
-                buffers,
-                arrow_data,
-                nodes,
-                offset,
-                is_little_endian,
-                write_keys,
-            ),
-            DataType::UInt32 => _write_dictionary::<u32>(
-                array,
-                buffers,
-                arrow_data,
-                nodes,
-                offset,
-                is_little_endian,
-                write_keys,
-            ),
-            DataType::UInt64 => _write_dictionary::<u64>(
-                array,
-                buffers,
-                arrow_data,
-                nodes,
-                offset,
-                is_little_endian,
-                write_keys,
-            ),
-            _ => unreachable!(),
-        },
+        DataType::Dictionary(key_type, _) => {
+            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+                _write_dictionary::<$T>(
+                    array,
+                    buffers,
+                    arrow_data,
+                    nodes,
+                    offset,
+                    is_little_endian,
+                    write_keys,
+                )
+            })
+        }
         _ => unreachable!(),
     }
 }
@@ -467,7 +429,9 @@ pub fn write(
                 true,
             );
         }
-        DataType::Union(_) => unimplemented!(),
+        DataType::Union(_, _, _) => {
+            write_union(array, buffers, arrow_data, nodes, offset, is_little_endian);
+        }
     }
 }
 
@@ -516,12 +480,14 @@ fn write_bitmap(
     match bitmap {
         Some(bitmap) => {
             assert_eq!(bitmap.len(), length);
-            if bitmap.offset() != 0 {
+            let (slice, slice_offset, _) = bitmap.as_slice();
+            if slice_offset != 0 {
                 // case where we can't slice the bitmap as the offsets are not multiple of 8
                 let bytes = Bitmap::from_trusted_len_iter(bitmap.iter());
-                write_bytes(bytes.as_slice(), buffers, arrow_data, offset)
+                let (slice, _, _) = bytes.as_slice();
+                write_bytes(slice, buffers, arrow_data, offset)
             } else {
-                write_bytes(bitmap.as_slice(), buffers, arrow_data, offset)
+                write_bytes(slice, buffers, arrow_data, offset)
             }
         }
         None => {
