@@ -1,8 +1,8 @@
 use futures::{pin_mut, Stream, StreamExt};
 use parquet2::{
     encoding::{bitpacking, hybrid_rle, uleb128, Encoding},
-    page::{DataPage, DataPageHeader, DataPageHeaderExt, FixedLenByteArrayPageDict},
-    read::{levels, StreamingIterator},
+    page::{DataPage, FixedLenByteArrayPageDict},
+    read::StreamingIterator,
 };
 
 use super::{ColumnChunkMetaData, ColumnDescriptor};
@@ -203,88 +203,39 @@ pub(crate) fn extend_from_page(
     assert_eq!(descriptor.max_rep_level(), 0);
     assert!(descriptor.max_def_level() <= 1);
     let is_optional = descriptor.max_def_level() == 1;
-    match page.header() {
-        DataPageHeader::V1(header) => {
-            assert_eq!(header.definition_level_encoding(), Encoding::Rle);
 
-            let (_, validity_buffer, values_buffer) =
-                levels::split_buffer_v1(page.buffer(), false, is_optional);
+    let (validity_buffer, values_buffer, version) = utils::split_buffer(page, is_optional);
 
-            match (page.encoding(), page.dictionary_page(), is_optional) {
-                (Encoding::PlainDictionary, Some(dict), true) => read_dict_buffer(
-                    validity_buffer,
-                    values_buffer,
-                    page.num_values() as u32,
-                    size,
-                    dict.as_any().downcast_ref().unwrap(),
-                    values,
-                    validity,
-                ),
-                (Encoding::Plain, None, true) => read_optional(
-                    validity_buffer,
-                    values_buffer,
-                    page.num_values() as u32,
-                    size,
-                    values,
-                    validity,
-                ),
-                (Encoding::Plain, None, false) => {
-                    read_required(page.buffer(), page.num_values() as u32, size, values)
-                }
-                _ => {
-                    return Err(utils::not_implemented(
-                        &page.encoding(),
-                        is_optional,
-                        page.dictionary_page().is_some(),
-                        "V1",
-                        "Binary",
-                    ))
-                }
-            }
+    match (page.encoding(), page.dictionary_page(), is_optional) {
+        (Encoding::PlainDictionary, Some(dict), true) => read_dict_buffer(
+            validity_buffer,
+            values_buffer,
+            page.num_values() as u32,
+            size,
+            dict.as_any().downcast_ref().unwrap(),
+            values,
+            validity,
+        ),
+        (Encoding::Plain, None, true) => read_optional(
+            validity_buffer,
+            values_buffer,
+            page.num_values() as u32,
+            size,
+            values,
+            validity,
+        ),
+        (Encoding::Plain, None, false) => {
+            read_required(page.buffer(), page.num_values() as u32, size, values)
         }
-        DataPageHeader::V2(header) => {
-            let def_level_buffer_length = header.definition_levels_byte_length as usize;
-
-            match (page.encoding(), page.dictionary_page(), is_optional) {
-                (Encoding::PlainDictionary, Some(dict), true) => {
-                    let (_, validity_buffer, values_buffer) =
-                        levels::split_buffer_v2(page.buffer(), 0, def_level_buffer_length);
-                    read_dict_buffer(
-                        validity_buffer,
-                        values_buffer,
-                        page.num_values() as u32,
-                        size,
-                        dict.as_any().downcast_ref().unwrap(),
-                        values,
-                        validity,
-                    )
-                }
-                (Encoding::Plain, None, true) => {
-                    let (_, validity_buffer, values_buffer) =
-                        levels::split_buffer_v2(page.buffer(), 0, def_level_buffer_length);
-                    read_optional(
-                        validity_buffer,
-                        values_buffer,
-                        page.num_values() as u32,
-                        size,
-                        values,
-                        validity,
-                    )
-                }
-                (Encoding::Plain, None, false) => {
-                    read_required(page.buffer(), page.num_values() as u32, size, values)
-                }
-                _ => {
-                    return Err(utils::not_implemented(
-                        &page.encoding(),
-                        is_optional,
-                        page.dictionary_page().is_some(),
-                        "V2",
-                        "Binary",
-                    ))
-                }
-            }
+        _ => {
+            return Err(utils::not_implemented(
+                &page.encoding(),
+                is_optional,
+                page.dictionary_page().is_some(),
+                version,
+                "FixedSizeBinary",
+            ))
         }
-    };
+    }
     Ok(())
 }
