@@ -11,8 +11,6 @@ mod utils;
 
 pub mod stream;
 
-use std::sync::Arc;
-
 use crate::array::*;
 use crate::bitmap::Bitmap;
 use crate::buffer::{Buffer, MutableBuffer};
@@ -105,43 +103,23 @@ pub fn can_encode(data_type: &DataType, encoding: Encoding) -> bool {
 
 /// Returns an iterator of compressed pages,
 pub fn array_to_pages(
-    array: Arc<dyn Array>,
+    array: &dyn Array,
     descriptor: ColumnDescriptor,
     options: WriteOptions,
     encoding: Encoding,
 ) -> Result<DynIter<'static, Result<CompressedPage>>> {
     match array.data_type() {
-        DataType::Dictionary(key, _) => match key.as_ref() {
-            DataType::Int8 => dictionary::array_to_pages::<i8>(
-                array.as_any().downcast_ref().unwrap(),
-                descriptor,
-                options,
-                encoding,
-            ),
-            DataType::Int16 => dictionary::array_to_pages::<i16>(
-                array.as_any().downcast_ref().unwrap(),
-                descriptor,
-                options,
-                encoding,
-            ),
-            DataType::Int32 => dictionary::array_to_pages::<i32>(
-                array.as_any().downcast_ref().unwrap(),
-                descriptor,
-                options,
-                encoding,
-            ),
-            DataType::Int64 => dictionary::array_to_pages::<i64>(
-                array.as_any().downcast_ref().unwrap(),
-                descriptor,
-                options,
-                encoding,
-            ),
-            other => Err(ArrowError::NotYetImplemented(format!(
-                "Writing parquet pages for data type {:?}",
-                other
-            ))),
-        },
-        _ => array_to_page(array.as_ref(), descriptor, options, encoding)
+        DataType::Dictionary(key_type, _) => {
+            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+                dictionary::array_to_pages::<$T>(
+                    array.as_any().downcast_ref().unwrap(),
+                    descriptor,
+                    options,
+                    encoding,
+                )
+            })
+        }
+        _ => array_to_page(array, descriptor, options, encoding)
             .map(|page| DynIter::new(std::iter::once(Ok(page)))),
     }
 }
@@ -478,376 +456,3 @@ fn nested_array_to_page(
         _ => todo!(),
     }
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    use crate::{error::Result, record_batch::RecordBatch};
-    use std::io::Cursor;
-
-    use super::super::tests::*;
-
-    fn round_trip(
-        column: usize,
-        nullable: bool,
-        nested: bool,
-        version: Version,
-        compression: Compression,
-        encoding: Encoding,
-    ) -> Result<()> {
-        let (array, statistics) = if nested {
-            (
-                pyarrow_nested_nullable(column),
-                pyarrow_nested_nullable_statistics(column),
-            )
-        } else if nullable {
-            (
-                pyarrow_nullable(column),
-                pyarrow_nullable_statistics(column),
-            )
-        } else {
-            (
-                pyarrow_required(column),
-                pyarrow_required_statistics(column),
-            )
-        };
-        let array: Arc<dyn Array> = array.into();
-
-        let field = Field::new("a1", array.data_type().clone(), nullable);
-        let schema = Schema::new(vec![field]);
-
-        let options = WriteOptions {
-            write_statistics: true,
-            compression,
-            version,
-        };
-
-        let parquet_schema = to_parquet_schema(&schema)?;
-
-        let iter = vec![RecordBatch::try_new(
-            Arc::new(schema.clone()),
-            vec![array.clone()],
-        )];
-
-        let row_groups =
-            RowGroupIterator::try_new(iter.into_iter(), &schema, options, vec![encoding])?;
-
-        let mut writer = Cursor::new(vec![]);
-        write_file(
-            &mut writer,
-            row_groups,
-            &schema,
-            parquet_schema,
-            options,
-            None,
-        )?;
-
-        let data = writer.into_inner();
-
-        let (result, stats) = read_column(&mut Cursor::new(data), 0, 0)?;
-        assert_eq!(array.as_ref(), result.as_ref());
-        assert_eq!(statistics.as_ref(), stats.as_ref());
-        Ok(())
-    }
-
-    #[test]
-    fn test_int64_optional_v1() -> Result<()> {
-        round_trip(
-            0,
-            true,
-            false,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_int64_required_v1() -> Result<()> {
-        round_trip(
-            0,
-            false,
-            false,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_int64_optional_v2() -> Result<()> {
-        round_trip(
-            0,
-            true,
-            false,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_int64_optional_v2_compressed() -> Result<()> {
-        round_trip(
-            0,
-            true,
-            false,
-            Version::V2,
-            Compression::Snappy,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_utf8_optional_v1() -> Result<()> {
-        round_trip(
-            2,
-            true,
-            false,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_utf8_required_v1() -> Result<()> {
-        round_trip(
-            2,
-            false,
-            false,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_utf8_optional_v2() -> Result<()> {
-        round_trip(
-            2,
-            true,
-            false,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_utf8_required_v2() -> Result<()> {
-        round_trip(
-            2,
-            false,
-            false,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_utf8_optional_v2_compressed() -> Result<()> {
-        round_trip(
-            2,
-            true,
-            false,
-            Version::V2,
-            Compression::Snappy,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_utf8_required_v2_compressed() -> Result<()> {
-        round_trip(
-            2,
-            false,
-            false,
-            Version::V2,
-            Compression::Snappy,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_bool_optional_v1() -> Result<()> {
-        round_trip(
-            3,
-            true,
-            false,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_bool_required_v1() -> Result<()> {
-        round_trip(
-            3,
-            false,
-            false,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_bool_optional_v2_uncompressed() -> Result<()> {
-        round_trip(
-            3,
-            true,
-            false,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_bool_required_v2_uncompressed() -> Result<()> {
-        round_trip(
-            3,
-            false,
-            false,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_bool_required_v2_compressed() -> Result<()> {
-        round_trip(
-            3,
-            false,
-            false,
-            Version::V2,
-            Compression::Snappy,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_list_int64_optional_v2() -> Result<()> {
-        round_trip(
-            0,
-            true,
-            true,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_list_int64_optional_v1() -> Result<()> {
-        round_trip(
-            0,
-            true,
-            true,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_list_bool_optional_v2() -> Result<()> {
-        round_trip(
-            4,
-            true,
-            true,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_list_bool_optional_v1() -> Result<()> {
-        round_trip(
-            4,
-            true,
-            true,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_list_utf8_optional_v2() -> Result<()> {
-        round_trip(
-            5,
-            true,
-            true,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_list_utf8_optional_v1() -> Result<()> {
-        round_trip(
-            5,
-            true,
-            true,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_list_large_binary_optional_v2() -> Result<()> {
-        round_trip(
-            6,
-            true,
-            true,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_list_large_binary_optional_v1() -> Result<()> {
-        round_trip(
-            6,
-            true,
-            true,
-            Version::V1,
-            Compression::Uncompressed,
-            Encoding::Plain,
-        )
-    }
-
-    #[test]
-    fn test_utf8_optional_v2_delta() -> Result<()> {
-        round_trip(
-            2,
-            true,
-            false,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::DeltaLengthByteArray,
-        )
-    }
-
-    #[test]
-    fn test_i32_optional_v2_dict() -> Result<()> {
-        round_trip(
-            6,
-            true,
-            false,
-            Version::V2,
-            Compression::Uncompressed,
-            Encoding::RleDictionary,
-        )
-    }
-}
