@@ -61,35 +61,3 @@ impl<O: Offset> Scalar for BinaryScalar<O> {
         }
     }
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[allow(clippy::eq_op)]
-    #[test]
-    fn equal() {
-        let a = BinaryScalar::<i32>::from(Some("a"));
-        let b = BinaryScalar::<i32>::from(None::<&str>);
-        assert_eq!(a, a);
-        assert_eq!(b, b);
-        assert!(a != b);
-        let b = BinaryScalar::<i32>::from(Some("b"));
-        assert!(a != b);
-        assert_eq!(b, b);
-    }
-
-    #[test]
-    fn basics() {
-        let a = BinaryScalar::<i32>::from(Some("a"));
-
-        assert_eq!(a.value(), b"a");
-        assert_eq!(a.data_type(), &DataType::Binary);
-        assert!(a.is_valid());
-
-        let a = BinaryScalar::<i64>::from(None::<&str>);
-
-        assert_eq!(a.data_type(), &DataType::LargeBinary);
-        assert!(!a.is_valid());
-    }
-}
